--	LibScriptLink
--	Implements a hyperlink scripting system that (hopefully) avoids taint.


local major, minor, addonTable = LibStub("LibBuilder-1.0"):GetLib()
if not addonTable.canLoad then return end


local Core = addonTable:NewClass("Core")

LibStub("AceHook-3.0"):Embed(Core)

Core.references = {}


function Core:ProcessLink(link)
	local capture = string.match(link, "ScriptLink:%l+:[^:|]+")
	
	if capture then
		local scriptType = string.match(capture, ":(%l+):")
		
		if scriptType == "ref" then
			local ref = string.match(capture, ":%l+:(.*)")
			
			local callback = self.references[ref]
			assert(callback)
			
			callback()		--	TODO: support modified clicks
			
			return true
		elseif scriptType == "inline" then
			local script = string.match(capture, ":%l+:(.*)")

			RunScript(script)
			return true
		else
			error()
		end
	else
		return false
	end
end

--	If SetItemRef doesn't know how to handle a link, it will send 
--	it to either HandleModifiedItemClick or ItemRefTooltip.SetHyperLink

--	Activates our hooks
function Core:HookAll()
	self:RawHook(ItemRefTooltip, "SetHyperlink", nil, true)
	self:SecureHook("HandleModifiedItemClick")
end


--	raw hook
function Core:SetHyperlink(...)
	local link = select(2, ...)
	
	local foundLink = self:ProcessLink(link)
	
	if foundLink then
		return			--	FIXME: do we need to return anything?
	else
		return self.hooks[ItemRefTooltip].SetHyperlink(...)
	end
end


--	secure hook
function Core:HandleModifiedItemClick(...)
	local link = ...
	
	self:ProcessLink(link)
	
	--	secure hook, so no point returning anything
end


function Core:RegisterCallback(callback)
	local newRef = self:NewReference()
	
	self.references[newRef] = callback
	
	return newRef
end


function Core:NewReference()
	local n = 1
	
	while self.references[tostring(n)] ~= nil do
		n = n + 1
	end
	--	 ref needs to be a string
	
	return tostring(n)
end
